/* -*- C++ -*-
 *
 * This file is a part of LEMON, a generic C++ optimization library
 *
 * Copyright (C) 2003-2006
 * Egervary Jeno Kombinatorikus Optimalizalasi Kutatocsoport
 * (Egervary Research Group on Combinatorial Optimization, EGRES).
 *
 * Permission to use, modify and distribute this software is granted
 * provided that this copyright notice appears in all copies. For
 * precise terms see the accompanying LICENSE file.
 *
 * This software is provided "AS IS" with no warranty of any kind,
 * express or implied, and with no claim as to its suitability for any
 * purpose.
 *
 */

#ifndef LEMON_MAPS_H
#define LEMON_MAPS_H

#include <iterator>
#include <functional>

#include <lemon/bits/utility.h>
#include <lemon/bits/traits.h>

///\file
///\ingroup maps
///\brief Miscellaneous property maps
///
///\todo This file has the same name as the concept file in concept/,
/// and this is not easily detectable in docs...

#include <map>

namespace lemon {

  /// \addtogroup maps
  /// @{

  /// Base class of maps.

  /// Base class of maps.
  /// It provides the necessary <tt>typedef</tt>s required by the map concept.
  template<typename K, typename T>
  class MapBase {
  public:
    ///\e
    typedef K Key;
    ///\e
    typedef T Value;
  };

  /// Null map. (a.k.a. DoNothingMap)

  /// If you have to provide a map only for its type definitions,
  /// or if you have to provide a writable map, but
  /// data written to it will sent to <tt>/dev/null</tt>...
  template<typename K, typename T>
  class NullMap : public MapBase<K, T> {
  public:
    typedef MapBase<K, T> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;
    
    /// Gives back a default constructed element.
    T operator[](const K&) const { return T(); }
    /// Absorbs the value.
    void set(const K&, const T&) {}
  };

  template <typename K, typename V> 
  NullMap<K, V> nullMap() {
    return NullMap<K, V>();
  }


  /// Constant map.

  /// This is a readable map which assigns a specified value to each key.
  /// In other aspects it is equivalent to the \ref NullMap.
  /// \todo set could be used to set the value.
  template<typename K, typename T>
  class ConstMap : public MapBase<K, T> {
  private:
    T v;
  public:

    typedef MapBase<K, T> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    /// Default constructor

    /// The value of the map will be uninitialized. 
    /// (More exactly it will be default constructed.)
    ConstMap() {}
    ///\e

    /// \param _v The initial value of the map.
    ///
    ConstMap(const T &_v) : v(_v) {}

    T operator[](const K&) const { return v; }
    void set(const K&, const T&) {}

    template<typename T1>
    struct rebind {
      typedef ConstMap<K, T1> other;
    };

    template<typename T1>
    ConstMap(const ConstMap<K, T1> &, const T &_v) : v(_v) {}
  };

  ///Returns a \ref ConstMap class

  ///This function just returns a \ref ConstMap class.
  ///\relates ConstMap
  template<typename K, typename V> 
  inline ConstMap<K, V> constMap(const V &v) {
    return ConstMap<K, V>(v);
  }


  //\todo to document later
  template<typename T, T v>
  struct Const { };

  //\todo to document later
  template<typename K, typename V, V v>
  class ConstMap<K, Const<V, v> > : public MapBase<K, V> {
  public:
    typedef MapBase<K, V> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ConstMap() { }
    V operator[](const K&) const { return v; }
    void set(const K&, const V&) { }
  };

  ///Returns a \ref ConstMap class

  ///This function just returns a \ref ConstMap class.
  ///\relates ConstMap
  template<typename K, typename V, V v> 
  inline ConstMap<K, Const<V, v> > constMap() {
    return ConstMap<K, Const<V, v> >();
  }

  /// \c std::map wrapper

  /// This is essentially a wrapper for \c std::map. With addition that
  /// you can specify a default value different from \c Value() .
  ///
  /// \todo Provide allocator parameter...
  template <typename K, typename T, typename Compare = std::less<K> >
  class StdMap : public std::map<K, T, Compare> {
    typedef std::map<K, T, Compare> parent;
    T v;
    typedef typename parent::value_type PairType;

  public:
    ///\e
    typedef K Key;
    ///\e
    typedef T Value;
    ///\e
    typedef T& Reference;
    ///\e
    typedef const T& ConstReference;


    StdMap() : v() {}
    /// Constructor with specified default value
    StdMap(const T& _v) : v(_v) {}

    /// \brief Constructs the map from an appropriate std::map.
    ///
    /// \warning Inefficient: copies the content of \c m !
    StdMap(const parent &m) : parent(m) {}
    /// \brief Constructs the map from an appropriate std::map, and explicitly
    /// specifies a default value.
    ///
    /// \warning Inefficient: copies the content of \c m !
    StdMap(const parent &m, const T& _v) : parent(m), v(_v) {}
    
    template<typename T1, typename Comp1>
    StdMap(const StdMap<Key, T1,Comp1> &m, const T &_v) { 
      //FIXME; 
    }

    Reference operator[](const Key &k) {
      return insert(PairType(k,v)).first -> second;
    }

    ConstReference operator[](const Key &k) const {
      typename parent::iterator i = lower_bound(k);
      if (i == parent::end() || parent::key_comp()(k, (*i).first))
	return v;
      return (*i).second;
    }
    void set(const Key &k, const T &t) {
      parent::operator[](k) = t;
    }

    /// Changes the default value of the map.
    /// \return Returns the previous default value.
    ///
    /// \warning The value of some keys (which has already been queried, but
    /// the value has been unchanged from the default) may change!
    T setDefault(const T &_v) { T old=v; v=_v; return old; }

    template<typename T1>
    struct rebind {
      typedef StdMap<Key, T1,Compare> other;
    };
  };

  /// @}

  /// \addtogroup map_adaptors
  /// @{

  /// \brief Identity mapping.
  ///
  /// This mapping gives back the given key as value without any
  /// modification. 
  template <typename T>
  class IdentityMap : public MapBase<T, T> {
  public:
    typedef MapBase<T, T> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    const T& operator[](const T& t) const {
      return t;
    }
  };

  ///Returns an \ref IdentityMap class

  ///This function just returns an \ref IdentityMap class.
  ///\relates IdentityMap
  template<typename T>
  inline IdentityMap<T> identityMap() {
    return IdentityMap<T>();
  }
  

  ///Convert the \c Value of a map to another type.

  ///This \ref concept::ReadMap "read only map"
  ///converts the \c Value of a maps to type \c T.
  ///Its \c Key is inherited from \c M.
  template <typename M, typename T> 
  class ConvertMap : public MapBase<typename M::Key, T> {
    const M& m;
  public:
    typedef MapBase<typename M::Key, T> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor

    ///Constructor
    ///\param _m is the underlying map
    ConvertMap(const M &_m) : m(_m) {};

    /// \brief The subscript operator.
    ///
    /// The subscript operator.
    /// \param k The key
    /// \return The target of the edge 
    Value operator[](const Key& k) const {return m[k];}
  };
  
  ///Returns an \ref ConvertMap class

  ///This function just returns an \ref ConvertMap class.
  ///\relates ConvertMap
  ///\todo The order of the template parameters are changed.
  template<typename T, typename M>
  inline ConvertMap<M, T> convertMap(const M &m) {
    return ConvertMap<M, T>(m);
  }

  ///Simple wrapping of the map

  ///This \ref concept::ReadMap "read only map" returns the simple
  ///wrapping of the given map. Sometimes the reference maps cannot be
  ///combined with simple read maps. This map adaptor wraps the given
  ///map to simple read map.
  template<typename M> 
  class SimpleMap : public MapBase<typename M::Key, typename M::Value> {
    const M& m;

  public:
    typedef MapBase<typename M::Key, typename M::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    SimpleMap(const M &_m) : m(_m) {};
    Value operator[](Key k) const {return m[k];}
  };

  ///Simple writeable wrapping of the map

  ///This \ref concept::ReadMap "read only map" returns the simple
  ///wrapping of the given map. Sometimes the reference maps cannot be
  ///combined with simple read-write maps. This map adaptor wraps the
  ///given map to simple read-write map.
  template<typename M> 
  class SimpleWriteMap : public MapBase<typename M::Key, typename M::Value> {
    M& m;

  public:
    typedef MapBase<typename M::Key, typename M::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    SimpleWriteMap(M &_m) : m(_m) {};
    Value operator[](Key k) const {return m[k];}
    void set(Key k, const Value& c) { m.set(k, c); }
  };

  ///Sum of two maps

  ///This \ref concept::ReadMap "read only map" returns the sum of the two
  ///given maps. Its \c Key and \c Value will be inherited from \c M1.
  ///The \c Key and \c Value of M2 must be convertible to those of \c M1.

  template<typename M1, typename M2> 
  class AddMap : public MapBase<typename M1::Key, typename M1::Value> {
    const M1& m1;
    const M2& m2;

  public:
    typedef MapBase<typename M1::Key, typename M1::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    AddMap(const M1 &_m1,const M2 &_m2) : m1(_m1), m2(_m2) {};
    Value operator[](Key k) const {return m1[k]+m2[k];}
  };
  
  ///Returns an \ref AddMap class

  ///This function just returns an \ref AddMap class.
  ///\todo How to call these type of functions?
  ///
  ///\relates AddMap
  ///\todo Wrong scope in Doxygen when \c \\relates is used
  template<typename M1, typename M2> 
  inline AddMap<M1, M2> addMap(const M1 &m1,const M2 &m2) {
    return AddMap<M1, M2>(m1,m2);
  }

  ///Shift a map with a constant.

  ///This \ref concept::ReadMap "read only map" returns the sum of the
  ///given map and a constant value.
  ///Its \c Key and \c Value is inherited from \c M.
  ///
  ///Actually,
  ///\code
  ///  ShiftMap<X> sh(x,v);
  ///\endcode
  ///is equivalent with
  ///\code
  ///  ConstMap<X::Key, X::Value> c_tmp(v);
  ///  AddMap<X, ConstMap<X::Key, X::Value> > sh(x,v);
  ///\endcode
  template<typename M, typename C = typename M::Value> 
  class ShiftMap : public MapBase<typename M::Key, typename M::Value> {
    const M& m;
    C v;
  public:
    typedef MapBase<typename M::Key, typename M::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor

    ///Constructor
    ///\param _m is the undelying map
    ///\param _v is the shift value
    ShiftMap(const M &_m, const C &_v ) : m(_m), v(_v) {};
    Value operator[](Key k) const {return m[k] + v;}
  };

  ///Shift a map with a constant.

  ///This \ref concept::ReadWriteMap "read-write map" returns the sum of the
  ///given map and a constant value. It makes also possible to write the map.
  ///Its \c Key and \c Value is inherited from \c M.
  ///
  ///Actually,
  ///\code
  ///  ShiftMap<X> sh(x,v);
  ///\endcode
  ///is equivalent with
  ///\code
  ///  ConstMap<X::Key, X::Value> c_tmp(v);
  ///  AddMap<X, ConstMap<X::Key, X::Value> > sh(x,v);
  ///\endcode
  template<typename M, typename C = typename M::Value> 
  class ShiftWriteMap : public MapBase<typename M::Key, typename M::Value> {
    M& m;
    C v;
  public:
    typedef MapBase<typename M::Key, typename M::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor

    ///Constructor
    ///\param _m is the undelying map
    ///\param _v is the shift value
    ShiftWriteMap(M &_m, const C &_v ) : m(_m), v(_v) {};
    Value operator[](Key k) const {return m[k] + v;}
    void set(Key k, const Value& c) { m.set(k, c - v); }
  };
  
  ///Returns an \ref ShiftMap class

  ///This function just returns an \ref ShiftMap class.
  ///\relates ShiftMap
  ///\todo A better name is required.
  template<typename M, typename C> 
  inline ShiftMap<M, C> shiftMap(const M &m,const C &v) {
    return ShiftMap<M, C>(m,v);
  }

  template<typename M, typename C> 
  inline ShiftWriteMap<M, C> shiftMap(M &m,const C &v) {
    return ShiftWriteMap<M, C>(m,v);
  }

  ///Difference of two maps

  ///This \ref concept::ReadMap "read only map" returns the difference
  ///of the values of the two
  ///given maps. Its \c Key and \c Value will be inherited from \c M1.
  ///The \c Key and \c Value of \c M2 must be convertible to those of \c M1.

  template<typename M1, typename M2> 
  class SubMap : public MapBase<typename M1::Key, typename M1::Value> {
    const M1& m1;
    const M2& m2;
  public:
    typedef MapBase<typename M1::Key, typename M1::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    SubMap(const M1 &_m1,const M2 &_m2) : m1(_m1), m2(_m2) {};
    Value operator[](Key k) const {return m1[k]-m2[k];}
  };
  
  ///Returns a \ref SubMap class

  ///This function just returns a \ref SubMap class.
  ///
  ///\relates SubMap
  template<typename M1, typename M2> 
  inline SubMap<M1, M2> subMap(const M1 &m1, const M2 &m2) {
    return SubMap<M1, M2>(m1, m2);
  }

  ///Product of two maps

  ///This \ref concept::ReadMap "read only map" returns the product of the
  ///values of the two
  ///given
  ///maps. Its \c Key and \c Value will be inherited from \c M1.
  ///The \c Key and \c Value of \c M2 must be convertible to those of \c M1.

  template<typename M1, typename M2> 
  class MulMap : public MapBase<typename M1::Key, typename M1::Value> {
    const M1& m1;
    const M2& m2;
  public:
    typedef MapBase<typename M1::Key, typename M1::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    MulMap(const M1 &_m1,const M2 &_m2) : m1(_m1), m2(_m2) {};
    Value operator[](Key k) const {return m1[k]*m2[k];}
  };
  
  ///Returns a \ref MulMap class

  ///This function just returns a \ref MulMap class.
  ///\relates MulMap
  template<typename M1, typename M2> 
  inline MulMap<M1, M2> mulMap(const M1 &m1,const M2 &m2) {
    return MulMap<M1, M2>(m1,m2);
  }
 
  ///Scales a maps with a constant.

  ///This \ref concept::ReadMap "read only map" returns the value of the
  ///given map multiplied from the left side with a constant value.
  ///Its \c Key and \c Value is inherited from \c M.
  ///
  ///Actually,
  ///\code
  ///  ScaleMap<X> sc(x,v);
  ///\endcode
  ///is equivalent with
  ///\code
  ///  ConstMap<X::Key, X::Value> c_tmp(v);
  ///  MulMap<X, ConstMap<X::Key, X::Value> > sc(x,v);
  ///\endcode
  template<typename M, typename C = typename M::Value> 
  class ScaleMap : public MapBase<typename M::Key, typename M::Value> {
    const M& m;
    C v;
  public:
    typedef MapBase<typename M::Key, typename M::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor

    ///Constructor
    ///\param _m is the undelying map
    ///\param _v is the scaling value
    ScaleMap(const M &_m, const C &_v ) : m(_m), v(_v) {};
    Value operator[](Key k) const {return v * m[k];}
  };

  ///Scales a maps with a constant.

  ///This \ref concept::ReadWriteMap "read-write map" returns the value of the
  ///given map multiplied from the left side with a constant value. It can
  ///be used as write map also if the given multiplier is not zero.
  ///Its \c Key and \c Value is inherited from \c M.
  template<typename M, typename C = typename M::Value> 
  class ScaleWriteMap : public MapBase<typename M::Key, typename M::Value> {
    M& m;
    C v;
  public:
    typedef MapBase<typename M::Key, typename M::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor

    ///Constructor
    ///\param _m is the undelying map
    ///\param _v is the scaling value
    ScaleWriteMap(M &_m, const C &_v ) : m(_m), v(_v) {};
    Value operator[](Key k) const {return v * m[k];}
    void set(Key k, const Value& c) { m.set(k, c / v);}
  };
  
  ///Returns an \ref ScaleMap class

  ///This function just returns an \ref ScaleMap class.
  ///\relates ScaleMap
  ///\todo A better name is required.
  template<typename M, typename C> 
  inline ScaleMap<M, C> scaleMap(const M &m,const C &v) {
    return ScaleMap<M, C>(m,v);
  }

  template<typename M, typename C> 
  inline ScaleWriteMap<M, C> scaleMap(M &m,const C &v) {
    return ScaleWriteMap<M, C>(m,v);
  }

  ///Quotient of two maps

  ///This \ref concept::ReadMap "read only map" returns the quotient of the
  ///values of the two
  ///given maps. Its \c Key and \c Value will be inherited from \c M1.
  ///The \c Key and \c Value of \c M2 must be convertible to those of \c M1.

  template<typename M1, typename M2> 
  class DivMap : public MapBase<typename M1::Key, typename M1::Value> {
    const M1& m1;
    const M2& m2;
  public:
    typedef MapBase<typename M1::Key, typename M1::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    DivMap(const M1 &_m1,const M2 &_m2) : m1(_m1), m2(_m2) {};
    Value operator[](Key k) const {return m1[k]/m2[k];}
  };
  
  ///Returns a \ref DivMap class

  ///This function just returns a \ref DivMap class.
  ///\relates DivMap
  template<typename M1, typename M2> 
  inline DivMap<M1, M2> divMap(const M1 &m1,const M2 &m2) {
    return DivMap<M1, M2>(m1,m2);
  }
  
  ///Composition of two maps

  ///This \ref concept::ReadMap "read only map" returns the composition of
  ///two
  ///given maps. That is to say, if \c m1 is of type \c M1 and \c m2 is
  ///of \c M2,
  ///then for
  ///\code
  ///  ComposeMap<M1, M2> cm(m1,m2);
  ///\endcode
  /// <tt>cm[x]</tt> will be equal to <tt>m1[m2[x]]</tt>
  ///
  ///Its \c Key is inherited from \c M2 and its \c Value is from
  ///\c M1.
  ///The \c M2::Value must be convertible to \c M1::Key.
  ///\todo Check the requirements.

  template <typename M1, typename M2> 
  class ComposeMap : public MapBase<typename M2::Key, typename M1::Value> {
    const M1& m1;
    const M2& m2;
  public:
    typedef MapBase<typename M2::Key, typename M1::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    ComposeMap(const M1 &_m1,const M2 &_m2) : m1(_m1), m2(_m2) {};
    
    typename MapTraits<M1>::ConstReturnValue
    operator[](Key k) const {return m1[m2[k]];}
  };
  ///Returns a \ref ComposeMap class

  ///This function just returns a \ref ComposeMap class.
  ///
  ///\relates ComposeMap
  template <typename M1, typename M2> 
  inline ComposeMap<M1, M2> composeMap(const M1 &m1,const M2 &m2) {
    return ComposeMap<M1, M2>(m1,m2);
  }
  
  ///Combines of two maps using an STL (binary) functor.

  ///Combines of two maps using an STL (binary) functor.
  ///
  ///
  ///This \ref concept::ReadMap "read only map" takes two maps and a
  ///binary functor and returns the composition of
  ///the two
  ///given maps unsing the functor. 
  ///That is to say, if \c m1 and \c m2 is of type \c M1 and \c M2
  ///and \c f is of \c F,
  ///then for
  ///\code
  ///  CombineMap<M1, M2,F,V> cm(m1,m2,f);
  ///\endcode
  /// <tt>cm[x]</tt> will be equal to <tt>f(m1[x],m2[x])</tt>
  ///
  ///Its \c Key is inherited from \c M1 and its \c Value is \c V.
  ///The \c M2::Value and \c M1::Value must be convertible to the corresponding
  ///input parameter of \c F and the return type of \c F must be convertible
  ///to \c V.
  ///\todo Check the requirements.

  template<typename M1, typename M2, typename F,
	   typename V = typename F::result_type,
	   typename NC = False> 
  class CombineMap : public MapBase<typename M1::Key, V> {
    const M1& m1;
    const M2& m2;
    F f;
  public:
    typedef MapBase<typename M1::Key, V> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    CombineMap(const M1 &_m1,const M2 &_m2,const F &_f)
      : m1(_m1), m2(_m2), f(_f) {};
    Value operator[](Key k) const {return f(m1[k],m2[k]);}
  };
  
  ///Returns a \ref CombineMap class

  ///This function just returns a \ref CombineMap class.
  ///
  ///Only the first template parameter (the value type) must be given.
  ///
  ///For example if \c m1 and \c m2 are both \c double valued maps, then 
  ///\code
  ///combineMap<double>(m1,m2,std::plus<double>)
  ///\endcode
  ///is equivalent with
  ///\code
  ///addMap(m1,m2)
  ///\endcode
  ///
  ///\relates CombineMap
  template<typename M1, typename M2, typename F, typename V> 
  inline CombineMap<M1, M2, F, V> 
  combineMap(const M1& m1,const M2& m2, const F& f) {
    return CombineMap<M1, M2, F, V>(m1,m2,f);
  }

  template<typename M1, typename M2, typename F> 
  inline CombineMap<M1, M2, F, typename F::result_type> 
  combineMap(const M1& m1, const M2& m2, const F& f) {
    return combineMap<M1, M2, F, typename F::result_type>(m1,m2,f);
  }

  template<typename M1, typename M2, typename K1, typename K2, typename V> 
  inline CombineMap<M1, M2, V (*)(K1, K2), V> 
  combineMap(const M1 &m1, const M2 &m2, V (*f)(K1, K2)) {
    return combineMap<M1, M2, V (*)(K1, K2), V>(m1,m2,f);
  }

  ///Negative value of a map

  ///This \ref concept::ReadMap "read only map" returns the negative
  ///value of the
  ///value returned by the
  ///given map. Its \c Key and \c Value will be inherited from \c M.
  ///The unary \c - operator must be defined for \c Value, of course.

  template<typename M> 
  class NegMap : public MapBase<typename M::Key, typename M::Value> {
    const M& m;
  public:
    typedef MapBase<typename M::Key, typename M::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    NegMap(const M &_m) : m(_m) {};
    Value operator[](Key k) const {return -m[k];}
  };
  
  ///Negative value of a map

  ///This \ref concept::ReadWriteMap "read-write map" returns the negative
  ///value of the value returned by the
  ///given map. Its \c Key and \c Value will be inherited from \c M.
  ///The unary \c - operator must be defined for \c Value, of course.

  template<typename M> 
  class NegWriteMap : public MapBase<typename M::Key, typename M::Value> {
    M& m;
  public:
    typedef MapBase<typename M::Key, typename M::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    NegWriteMap(M &_m) : m(_m) {};
    Value operator[](Key k) const {return -m[k];}
    void set(Key k, const Value& v) { m.set(k, -v); }
  };

  ///Returns a \ref NegMap class

  ///This function just returns a \ref NegMap class.
  ///\relates NegMap
  template <typename M> 
  inline NegMap<M> negMap(const M &m) {
    return NegMap<M>(m);
  }

  template <typename M> 
  inline NegWriteMap<M> negMap(M &m) {
    return NegWriteMap<M>(m);
  }

  ///Absolute value of a map

  ///This \ref concept::ReadMap "read only map" returns the absolute value
  ///of the
  ///value returned by the
  ///given map. Its \c Key and \c Value will be inherited
  ///from <tt>M</tt>. <tt>Value</tt>
  ///must be comparable to <tt>0</tt> and the unary <tt>-</tt>
  ///operator must be defined for it, of course.
  ///
  ///\bug We need a unified way to handle the situation below:
  ///\code
  ///  struct _UnConvertible {};
  ///  template<class A> inline A t_abs(A a) {return _UnConvertible();}
  ///  template<> inline int t_abs<>(int n) {return abs(n);}
  ///  template<> inline long int t_abs<>(long int n) {return labs(n);}
  ///  template<> inline long long int t_abs<>(long long int n) {return ::llabs(n);}
  ///  template<> inline float t_abs<>(float n) {return fabsf(n);}
  ///  template<> inline double t_abs<>(double n) {return fabs(n);}
  ///  template<> inline long double t_abs<>(long double n) {return fabsl(n);}
  ///\endcode
  

  template<typename M> 
  class AbsMap : public MapBase<typename M::Key, typename M::Value> {
    const M& m;
  public:
    typedef MapBase<typename M::Key, typename M::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    AbsMap(const M &_m) : m(_m) {};
    Value operator[](Key k) const {
      Value tmp = m[k]; 
      return tmp >= 0 ? tmp : -tmp;
    }

  };
  
  ///Returns a \ref AbsMap class

  ///This function just returns a \ref AbsMap class.
  ///\relates AbsMap
  template<typename M> 
  inline AbsMap<M> absMap(const M &m) {
    return AbsMap<M>(m);
  }

  ///Converts an STL style functor to a map

  ///This \ref concept::ReadMap "read only map" returns the value
  ///of a
  ///given map.
  ///
  ///Template parameters \c K and \c V will become its
  ///\c Key and \c Value. They must be given explicitely
  ///because a functor does not provide such typedefs.
  ///
  ///Parameter \c F is the type of the used functor.
  

  template<typename F, 
	   typename K = typename F::argument_type, 
	   typename V = typename F::result_type,
	   typename NC = False> 
  class FunctorMap : public MapBase<K, V> {
    F f;
  public:
    typedef MapBase<K, V> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    FunctorMap(const F &_f) : f(_f) {}

    Value operator[](Key k) const { return f(k);}
  };
  
  ///Returns a \ref FunctorMap class

  ///This function just returns a \ref FunctorMap class.
  ///
  ///The third template parameter isn't necessary to be given.
  ///\relates FunctorMap
  template<typename K, typename V, typename F> inline 
  FunctorMap<F, K, V> functorMap(const F &f) {
    return FunctorMap<F, K, V>(f);
  }

  template <typename F> inline 
  FunctorMap<F, typename F::argument_type, typename F::result_type> 
  functorMap(const F &f) {
    return FunctorMap<F, typename F::argument_type, 
      typename F::result_type>(f);
  }

  template <typename K, typename V> inline 
  FunctorMap<V (*)(K), K, V> functorMap(V (*f)(K)) {
    return FunctorMap<V (*)(K), K, V>(f);
  }


  ///Converts a map to an STL style (unary) functor

  ///This class Converts a map to an STL style (unary) functor.
  ///that is it provides an <tt>operator()</tt> to read its values.
  ///
  ///For the sake of convenience it also works as
  ///a ususal \ref concept::ReadMap "readable map",
  ///i.e. <tt>operator[]</tt> and the \c Key and \c Value typedefs also exist.

  template <typename M> 
  class MapFunctor : public MapBase<typename M::Key, typename M::Value> {
    const M& m;
  public:
    typedef MapBase<typename M::Key, typename M::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///\e
    typedef typename M::Key argument_type;
    ///\e
    typedef typename M::Value result_type;

    ///Constructor
    MapFunctor(const M &_m) : m(_m) {};
    ///Returns a value of the map
    Value operator()(Key k) const {return m[k];}
    ///\e
    Value operator[](Key k) const {return m[k];}
  };
  
  ///Returns a \ref MapFunctor class

  ///This function just returns a \ref MapFunctor class.
  ///\relates MapFunctor
  template<typename M> 
  inline MapFunctor<M> mapFunctor(const M &m) {
    return MapFunctor<M>(m);
  }

  ///Applies all map setting operations to two maps

  ///This map has two \ref concept::ReadMap "readable map"
  ///parameters and each read request will be passed just to the
  ///first map. This class is the just readable map type of the ForkWriteMap.
  ///
  ///The \c Key and \c Value will be inherited from \c M1.
  ///The \c Key and \c Value of M2 must be convertible from those of \c M1.

  template<typename  M1, typename M2> 
  class ForkMap : public MapBase<typename M1::Key, typename M1::Value> {
    const M1& m1;
    const M2& m2;
  public:
    typedef MapBase<typename M1::Key, typename M1::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    ForkMap(const M1 &_m1, const M2 &_m2) : m1(_m1), m2(_m2) {};
    Value operator[](Key k) const {return m1[k];}
  };


  ///Applies all map setting operations to two maps

  ///This map has two \ref concept::WriteMap "writable map"
  ///parameters and each write request will be passed to both of them.
  ///If \c M1 is also \ref concept::ReadMap "readable",
  ///then the read operations will return the
  ///corresponding values of \c M1.
  ///
  ///The \c Key and \c Value will be inherited from \c M1.
  ///The \c Key and \c Value of M2 must be convertible from those of \c M1.

  template<typename  M1, typename M2> 
  class ForkWriteMap : public MapBase<typename M1::Key, typename M1::Value> {
    M1& m1;
    M2& m2;
  public:
    typedef MapBase<typename M1::Key, typename M1::Value> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    ///Constructor
    ForkWriteMap(M1 &_m1, M2 &_m2) : m1(_m1), m2(_m2) {};
    Value operator[](Key k) const {return m1[k];}
    void set(Key k, const Value &v) {m1.set(k,v); m2.set(k,v);}
  };
  
  ///Returns an \ref ForkMap class

  ///This function just returns an \ref ForkMap class.
  ///\todo How to call these type of functions?
  ///
  ///\relates ForkMap
  ///\todo Wrong scope in Doxygen when \c \\relates is used
  template <typename M1, typename M2> 
  inline ForkMap<M1, M2> forkMap(const M1 &m1, const M2 &m2) {
    return ForkMap<M1, M2>(m1,m2);
  }

  template <typename M1, typename M2> 
  inline ForkWriteMap<M1, M2> forkMap(M1 &m1, M2 &m2) {
    return ForkWriteMap<M1, M2>(m1,m2);
  }


  
  /* ************* BOOL MAPS ******************* */
  
  ///Logical 'not' of a map
  
  ///This bool \ref concept::ReadMap "read only map" returns the 
  ///logical negation of
  ///value returned by the
  ///given map. Its \c Key and will be inherited from \c M,
  ///its Value is <tt>bool</tt>.

  template <typename M> 
  class NotMap : public MapBase<typename M::Key, bool> {
    const M& m;
  public:
    typedef MapBase<typename M::Key, bool> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    /// Constructor
    NotMap(const M &_m) : m(_m) {};
    Value operator[](Key k) const {return !m[k];}
  };

  ///Logical 'not' of a map with writing possibility
  
  ///This bool \ref concept::ReadWriteMap "read-write map" returns the 
  ///logical negation of value returned by the given map. It is setted
  ///then the negation of the value be setted to the original map.
  ///Its \c Key and will be inherited from \c M,
  ///its Value is <tt>bool</tt>.
  template <typename M> 
  class NotWriteMap : public MapBase<typename M::Key, bool> {
    M& m;
  public:
    typedef MapBase<typename M::Key, bool> Parent;
    typedef typename Parent::Key Key;
    typedef typename Parent::Value Value;

    /// Constructor
    NotWriteMap(M &_m) : m(_m) {};
    Value operator[](Key k) const {return !m[k];}
    void set(Key k, bool v) { m.set(k, !v); }
  };
  
  ///Returns a \ref NotMap class
  
  ///This function just returns a \ref NotMap class.
  ///\relates NotMap
  template <typename M> 
  inline NotMap<M> notMap(const M &m) {
    return NotMap<M>(m);
  }

  template <typename M> 
  inline NotWriteMap<M> notMap(M &m) {
    return NotWriteMap<M>(m);
  }

  namespace _maps_bits {
    template <typename Value>
    struct Identity {
      typedef Value argument_type;
      typedef Value result_type;
      Value operator()(const Value& val) {
	return val;
      }
    };
  }
  

  /// \brief Writable bool map for store each true assigned elements.
  ///
  /// Writable bool map for store each true assigned elements. It will
  /// copies all the true setted keys to the given iterator.
  ///
  /// \note The container of the iterator should contain space 
  /// for each element.
  ///
  /// The next example shows how can you write the nodes directly
  /// to the standard output.
  ///\code
  /// typedef IdMap<UGraph, UEdge> UEdgeIdMap;
  /// UEdgeIdMap uedgeId(ugraph);
  ///
  /// typedef MapFunctor<UEdgeIdMap> UEdgeIdFunctor;
  /// UEdgeIdFunctor uedgeIdFunctor(uedgeId);
  ///
  /// StoreBoolMap<ostream_iterator<int>, UEdgeIdFunctor> 
  ///   writerMap(ostream_iterator<int>(cout, " "), uedgeIdFunctor);
  ///
  /// prim(ugraph, cost, writerMap);
  ///\endcode
  template <typename _Iterator, 
            typename _Functor = 
            _maps_bits::Identity<typename std::iterator_traits<_Iterator>::value_type> >
  class StoreBoolMap {
  public:
    typedef _Iterator Iterator;

    typedef typename _Functor::argument_type Key;
    typedef bool Value;

    typedef _Functor Functor;

    /// Constructor
    StoreBoolMap(Iterator it, const Functor& functor = Functor()) 
      : _begin(it), _end(it), _functor(functor) {}

    /// Gives back the given first setted iterator.
    Iterator begin() const {
      return _begin;
    }
 
    /// Gives back the iterator after the last setted.
    Iterator end() const {
      return _end;
    }

    /// Setter function of the map
    void set(const Key& key, Value value) {
      if (value) {
	*_end++ = _functor(key);
      }
    }
    
  private:
    Iterator _begin, _end;
    Functor _functor;
  };

  /// \brief Writable bool map for store each true assigned elements in 
  /// a back insertable container.
  ///
  /// Writable bool map for store each true assigned elements in a back 
  /// insertable container. It will push back all the true setted keys into
  /// the container. It can be used to retrieve the items into a standard
  /// container. The next example shows how can you store the undirected
  /// edges in a vector with prim algorithm.
  ///
  ///\code
  /// vector<UEdge> span_tree_uedges;
  /// BackInserterBoolMap<vector<UEdge> > inserter_map(span_tree_uedges);
  /// prim(ugraph, cost, inserter_map);
  ///\endcode
  template <typename Container,
            typename Functor =
            _maps_bits::Identity<typename Container::value_type> >
  class BackInserterBoolMap {
  public:
    typedef typename Container::value_type Key;
    typedef bool Value;

    /// Constructor
    BackInserterBoolMap(Container& _container, 
                        const Functor& _functor = Functor()) 
      : container(_container), functor(_functor) {}

    /// Setter function of the map
    void set(const Key& key, Value value) {
      if (value) {
	container.push_back(functor(key));
      }
    }
    
  private:
    Container& container;
    Functor functor;
  };

  /// \brief Writable bool map for store each true assigned elements in 
  /// a front insertable container.
  ///
  /// Writable bool map for store each true assigned elements in a front 
  /// insertable container. It will push front all the true setted keys into
  /// the container. For example see the BackInserterBoolMap.
  template <typename Container,
            typename Functor =
            _maps_bits::Identity<typename Container::value_type> >
  class FrontInserterBoolMap {
  public:
    typedef typename Container::value_type Key;
    typedef bool Value;

    /// Constructor
    FrontInserterBoolMap(Container& _container,
                         const Functor& _functor = Functor()) 
      : container(_container), functor(_functor) {}

    /// Setter function of the map
    void set(const Key& key, Value value) {
      if (value) {
	container.push_front(key);
      }
    }
    
  private:
    Container& container;    
    Functor functor;
  };

  /// \brief Writable bool map for store each true assigned elements in 
  /// an insertable container.
  ///
  /// Writable bool map for store each true assigned elements in an 
  /// insertable container. It will insert all the true setted keys into
  /// the container. If you want to store the cut edges of the strongly
  /// connected components in a set you can use the next code:
  ///
  ///\code
  /// set<Edge> cut_edges;
  /// InserterBoolMap<set<Edge> > inserter_map(cut_edges);
  /// stronglyConnectedCutEdges(graph, cost, inserter_map);
  ///\endcode
  template <typename Container,
            typename Functor =
            _maps_bits::Identity<typename Container::value_type> >
  class InserterBoolMap {
  public:
    typedef typename Container::value_type Key;
    typedef bool Value;

    /// Constructor
    InserterBoolMap(Container& _container, typename Container::iterator _it,
                    const Functor& _functor = Functor()) 
      : container(_container), it(_it), functor(_functor) {}

    /// Constructor
    InserterBoolMap(Container& _container, const Functor& _functor = Functor())
      : container(_container), it(_container.end()), functor(_functor) {}

    /// Setter function of the map
    void set(const Key& key, Value value) {
      if (value) {
	it = container.insert(it, key);
        ++it;
      }
    }
    
  private:
    Container& container;
    typename Container::iterator it;
    Functor functor;
  };

  /// \brief Fill the true setted elements with a given value.
  ///
  /// Writable bool map for fill the true setted elements with a given value.
  /// The value can be setted 
  /// the container.
  ///
  /// The next code finds the connected components of the undirected graph
  /// and stores it in the \c comp map:
  ///\code
  /// typedef UGraph::NodeMap<int> ComponentMap;
  /// ComponentMap comp(ugraph);
  /// typedef FillBoolMap<UGraph::NodeMap<int> > ComponentFillerMap;
  /// ComponentFillerMap filler(comp, 0);
  ///
  /// Dfs<UGraph>::DefProcessedMap<ComponentFillerMap>::Create dfs(ugraph);
  /// dfs.processedMap(filler);
  /// dfs.init();
  /// for (NodeIt it(ugraph); it != INVALID; ++it) {
  ///   if (!dfs.reached(it)) {
  ///     dfs.addSource(it);
  ///     dfs.start();
  ///     ++filler.fillValue();
  ///   }
  /// }
  ///\endcode

  template <typename Map>
  class FillBoolMap {
  public:
    typedef typename Map::Key Key;
    typedef bool Value;

    /// Constructor
    FillBoolMap(Map& _map, const typename Map::Value& _fill) 
      : map(_map), fill(_fill) {}

    /// Constructor
    FillBoolMap(Map& _map) 
      : map(_map), fill() {}

    /// Gives back the current fill value
    const typename Map::Value& fillValue() const {
      return fill;
    } 

    /// Gives back the current fill value
    typename Map::Value& fillValue() {
      return fill;
    } 

    /// Sets the current fill value
    void fillValue(const typename Map::Value& _fill) {
      fill = _fill;
    } 

    /// Setter function of the map
    void set(const Key& key, Value value) {
      if (value) {
	map.set(key, fill);
      }
    }
    
  private:
    Map& map;
    typename Map::Value fill;
  };


  /// \brief Writable bool map which stores for each true assigned elements  
  /// the setting order number.
  ///
  /// Writable bool map which stores for each true assigned elements  
  /// the setting order number. It make easy to calculate the leaving
  /// order of the nodes in the \ref dfs "Dfs" algorithm.
  ///
  ///\code
  /// typedef Graph::NodeMap<int> OrderMap;
  /// OrderMap order(graph);
  /// typedef SettingOrderBoolMap<OrderMap> OrderSetterMap;
  /// OrderSetterMap setter(order);
  /// Dfs<Graph>::DefProcessedMap<OrderSetterMap>::Create dfs(graph);
  /// dfs.processedMap(setter);
  /// dfs.init();
  /// for (NodeIt it(graph); it != INVALID; ++it) {
  ///   if (!dfs.reached(it)) {
  ///     dfs.addSource(it);
  ///     dfs.start();
  ///   }
  /// }
  ///\endcode
  ///
  /// The discovering order can be stored a little harder because the
  /// ReachedMap should be readable in the dfs algorithm but the setting
  /// order map is not readable. Now we should use the fork map:
  ///
  ///\code
  /// typedef Graph::NodeMap<int> OrderMap;
  /// OrderMap order(graph);
  /// typedef SettingOrderBoolMap<OrderMap> OrderSetterMap;
  /// OrderSetterMap setter(order);
  /// typedef Graph::NodeMap<bool> StoreMap;
  /// StoreMap store(graph);
  ///
  /// typedef ForkWriteMap<StoreMap, OrderSetterMap> ReachedMap;
  /// ReachedMap reached(store, setter);
  ///
  /// Dfs<Graph>::DefReachedMap<ReachedMap>::Create dfs(graph);
  /// dfs.reachedMap(reached);
  /// dfs.init();
  /// for (NodeIt it(graph); it != INVALID; ++it) {
  ///   if (!dfs.reached(it)) {
  ///     dfs.addSource(it);
  ///     dfs.start();
  ///   }
  /// }
  ///\endcode
  template <typename Map>
  class SettingOrderBoolMap {
  public:
    typedef typename Map::Key Key;
    typedef bool Value;

    /// Constructor
    SettingOrderBoolMap(Map& _map) 
      : map(_map), counter(0) {}

    /// Number of setted keys.
    int num() const {
      return counter;
    }

    /// Setter function of the map
    void set(const Key& key, Value value) {
      if (value) {
	map.set(key, counter++);
      }
    }
    
  private:
    Map& map;
    int counter;
  };

  /// @}
}

#endif // LEMON_MAPS_H
